#!/bin/bash
TARGET=undefined
TARGET_UNCAPITALIZED=undefined
DEF_DIRECTORY=/opt/mvIMPACT_Acquire
PRODUCT=mvGenTL-Acquire
API=mvIMPACT_Acquire
TARNAME=mvGenTL_Acquire
ABI_POSTFIX=_ABI2
GENICAM_VERSION=3_0
LINK=mvimpact-acquire
USER="$(whoami)"
ARM_ARCHITECTURE="$(uname -m)"
GEV_SUPPORT=unknown
U3V_SUPPORT=unknown

function createSoftlink {
    if [ ! -e "$1/$2" ]; then
        echo "Error: File "$1/$2" does not exist, softlink cannot be created! "
        exit 1
    fi
    if [ -e "$1/$3" ]; then
        rm -rf "$1/$3" >/dev/null 2>&1
    fi
    if ! [ -L "$1/$3" ]; then
        ln -fs $2 "$1/$3" >/dev/null 2>&1
        if ! [ -L "$1/$3" ]; then
            sudo ln -fs $2 "$1/$3" >/dev/null 2>&1
            if ! [ -L "$1/$3" ]; then
                echo "Error: Could not create softlink $1/$3, even with sudo!"
                exit 1
            fi
        fi
    fi
}

# Define the users real name if possible, to prevent accidental mvIA root ownership if script is invoked with sudo
if [ "$(which logname)" == "" ] ; then
    USER=$(whoami)
else
    if [ "$(logname 2>&1 | grep -c logname:)" == "1" ] ; then
        USER=$(whoami)
    else
        USER=$(logname)
    fi
fi


if [ "$1" == "-h" ] || [ "$1" == "--help" ]; then
   echo
   echo 'Installation script for the '$PRODUCT' driver.'
   echo
   echo "The default installation path is: "$DEF_DIRECTORY
   echo "Usage:		./install_mvGenTL_Acquire.sh [OPTION] ... "
   echo "Example:	./install_mvGenTL_Acquire.sh -p /myPath"
   echo 
   echo "Arguments:"
   echo "-h	--help		This help print."
   echo "-p	--path		Set the directory where the files shall be installed."
   echo
   exit
fi   

# Print out ASCII-Art Logo.
clear;
echo ""
echo ""
echo ""
echo ""
echo "                                ===     ===    .MMMO                             "
echo "                                 ==+    ==     M         ,MMM   ?M MM,           "
echo "                                 .==   .=+     M  MMM   M    M   M   M           "
echo "                                  ==+  ==.     M    M   M ^^^    M   M           "
echo "             ..                   .== ,==       MMMM    'MMMM    M   M           "
echo "   MMMM   DMMMMMM      MMMMMM      =====                                         "
echo "   MMMM MMMMMMMMMMM :MMMMMMMMMM     ====          MMMMMMMMMMMM   MMM             "
echo "   MMMMMMMMMMMMMMMMMMMMMMMMMMMMM                 MMMMMMMMMMMM   MMM              "
echo "   MMMMMMM   .MMMMMMMM    MMMMMM                     MMM       MMM               "
echo "   MMMMM.      MMMMMM      MMMMM                    MM7       MMM                "
echo "   MMMMM       MMMMM       MMMMM                   MMM       IMM                 "
echo "   MMMMM       MMMMM       MMMMM                  MMM       MMMMMMMMMM           "
echo "   MMMMM       MMMMM       MMMMM                                                 "
echo "   MMMMM       MMMMM       MMMMM       M     MMM    MM    M   M  MMM  MMMM   MMMM"
echo "   MMMMM       MMMMM       MMMMM      M M   M   M  M   M  M   M   M   M   M  M   "
echo "   MMMMM       MMMMM       MMMMM     M   M  M      M   M  M   M   M   MMM,   MMM "
echo "   MMMMM       MMMMM       MMMMM     MMMMM  M   M  M  ,M  M   M   M   M   M  M   "
echo "                                     M   M  'MMM'   MMMM, 'MMM'  MMM  M   M  MMMM"
echo "==================================================================================" 
sleep 1

# Check if the user did specify that we shall use a specific directory instead of DEF_DIRECTORY
if [ "$1" == "-p" ] || [ "$1" == "--path" ] ; then
    if [ $(echo "$2") ] ; then
      DEF_DIRECTORY=$2
    else
      echo
      echo "WARNING: Path option used with no defined path, will use: $DEF_DIRECTORY directory"
    fi
else
   echo
   echo "No target directory specified, default directory: $DEF_DIRECTORY will be used..."
fi

# Get the intended target platform 
if [ "$( ls | grep -c 'mvGenTL_Acquire.*\.tgz' )" != "0" ] ; then
  TARNAME=`ls mvGenTL_Acquire*.tgz|tail -1 | sed -e s/\\.tgz//`
  if [ "$(echo $TARNAME | grep -c ARMhf)" != "0" ]; then
    TARGET="ARMhf"
    TARGET_UNCAPITALIZED="armhf"
  elif [ "$(echo $TARNAME | grep -c ARMsf)" != "0" ]; then
    TARGET="ARMsf"
    TARGET_UNCAPITALIZED="armsf"
  else
    echo "Error: Could not determine target architecture from file name."
    echo "In case the file been renamed, please revert to original name."
    echo "Terminating this installation script..."
    echo
    exit 1
  fi 
fi

# Get the source directory (the directory where the files for the installation are) and cd to it
# (The script file must be in the same directory as the source TGZ) !!!
if which dirname >/dev/null; then
    SCRIPTSOURCEDIR="$PWD/$(dirname $0)"
fi
if [ "$SCRIPTSOURCEDIR" != "$PWD" ]; then
   if [ "$SCRIPTSOURCEDIR" == "" ] || [ "$SCRIPTSOURCEDIR" == "." ]; then
      SCRIPTSOURCEDIR="$PWD"
   fi
   cd "$SCRIPTSOURCEDIR"
fi

# Set variables for GenICam and mvIMPACT_acquire for later use
if grep -q '/etc/ld.so.conf.d/' /etc/ld.so.conf; then
   GENICAM_LDSOCONF_FILE=/etc/ld.so.conf.d/genicam.conf
   ACQUIRE_LDSOCONF_FILE=/etc/ld.so.conf.d/acquire.conf
   #sudo rm -f $GENICAM_LDSOCONF_FILE; sudo touch $GENICAM_LDSOCONF_FILE
   #sudo rm -f $ACQUIRE_LDSOCONF_FILE; sudo touch $ACQUIRE_LDSOCONF_FILE
else
   GENICAM_LDSOCONF_FILE=/etc/ld.so.conf
   ACQUIRE_LDSOCONF_FILE=/etc/ld.so.conf
fi

# Make sure the environment variables are set at the next boot as well
if grep -q '/etc/profile.d/' /etc/profile; then
   GENICAM_EXPORT_FILE=/etc/profile.d/genicam.sh
   ACQUIRE_EXPORT_FILE=/etc/profile.d/acquire.sh
   #sudo rm -f $GENICAM_EXPORT_FILE; sudo touch $GENICAM_EXPORT_FILE
   #sudo rm -f $ACQUIRE_EXPORT_FILE; sudo touch $ACQUIRE_EXPORT_FILE
else
   GENICAM_EXPORT_FILE=/etc/profile
   ACQUIRE_EXPORT_FILE=/etc/profile
fi

# Get driver name, version, file
if [ "$( ls | grep -c 'mvGenTL_Acquire.*\.tgz' )" != "0" ] ; then
  TARNAME=`ls mvGenTL_Acquire*.tgz | tail -n 1 | sed -e s/\\.tgz//`
  TARFILE=`ls mvGenTL_Acquire*.tgz | tail -n 1`
  VERSION=`ls mvGenTL_Acquire*.tgz | tail -n 1 | sed -e s/\\mvGenTL_Acquire// | sed -e s/\\-$TARGET// | sed -e s/\\_gnueabi-// | sed -e s/\\.tgz//` 
  ACT2=$API-$VERSION
  ACT=$API-$TARGET-$VERSION
fi

# A quick check whether the Version has a correct format (due to other files being in the same directory..?)
if [ "$(echo $VERSION | grep -c '^[0-9]\{1,2\}\.[0-9]\{1,2\}\.[0-9]\{1,2\}')" == "0" ]; then
  echo "-----------------------------------------------------------------------------------"
  echo "  ABORTING: Script did not determine a valid mvIMPACT Acquire *.tgz file!  " 
  echo "-----------------------------------------------------------------------------------"
  echo "  This script could not extract a valid version number from the *.tgz file"
  echo "  This script determined $TARFILE as the file containing the installation data."
  echo "  It is recommended that only this script and the correct *.tgz file reside in this directory."
  echo "  Please remove all other files and try again."
  exit
fi

# A quick check whether the user has been determined
if [ "$USER" == "" ]; then
  echo "-----------------------------------------------------------------------------------"
  echo "  ABORTING: Script could not determine a valid user!  " 
  echo "-----------------------------------------------------------------------------------"
  echo "  This script could not determine the user of this shell"
  echo "  Please make sure this is a valid login shell!"
  exit
fi

YES_NO=
# Here we will ask the user if we shall start the installation process
echo
echo   "-----------------------------------------------------------------------------------"
echo   "Configuration:"
echo   "-----------------------------------------------------------------------------------"
echo
echo   "Installation for user:          "$USER
echo   "Installation directory:         "$DEF_DIRECTORY
echo   "Source directory:               "$(echo $SCRIPTSOURCEDIR | sed -e 's/\/\.//')
echo   "Version:                        "$VERSION
echo   "Platform:                       "$TARGET
echo   "TAR-File:                       "$TARFILE
echo
echo   "ldconfig:"
echo   "GenICam:                        "$GENICAM_LDSOCONF_FILE
echo   "mvIMPACT_acquire:               "$ACQUIRE_LDSOCONF_FILE
echo
echo   "Exports:"
echo   "GenICam:                        "$GENICAM_EXPORT_FILE
echo   "mvIMPACT_acquire:               "$ACQUIRE_EXPORT_FILE
echo 
echo   "-----------------------------------------------------------------------------------"
echo
echo "Do you want to continue (default is 'yes')?"
echo "Hit 'n' + <Enter> for 'no', or just <Enter> for 'yes'."
read YES_NO

# If the user is choosing no, we will abort the installation, else we will start the process.
if [ "$YES_NO" == "n" ] || [ "$YES_NO" == "N" ]; then
  echo "Quit!"
  exit
fi
 
# First of all ask whether to dispose of the old mvIMPACT Acquire installation
if [ "$MVIMPACT_ACQUIRE_DIR" != "" ]; then
  echo "Do you want to keep previous installation (default is 'yes')?"
  echo "If you select no, mvIMPACT Acquire will be removed for ALL installed Products!"
  echo "Hit 'n' + <Enter> for 'no', or just <Enter> for 'yes'."
if [ "$USE_DEFAULTS" == "NO" ] ; then
  read YES_NO
else
  YES_NO=""
fi
  if [ "$YES_NO" == "n" ] || [ "$YES_NO" == "N" ]; then
    sudo rm -f /usr/bin/mvDeviceConfigure >/dev/null 2>&1
    sudo rm -f /usr/bin/mvIPConfigure >/dev/null 2>&1
    sudo rm -f /usr/bin/wxPropView >/dev/null 2>&1
    sudo rm -rf $MVIMPACT_ACQUIRE_DIR >/dev/null 2>&1
    if [ $? == 0 ]; then
      echo "Previous mvIMPACT Acquire Installation ($MVIMPACT_ACQUIRE_DIR) removed successfully!"
    else
      echo "Error removing previous mvIMPACT Acquire Installation ($MVIMPACT_ACQUIRE_DIR)!"
      echo "$?"
    fi
  else
    echo "Previous mvIMPACT Acquire Installation ($MVIMPACT_ACQUIRE_DIR) NOT removed!"
  fi
fi
 
 # Determine whether mvGenTL_Acquire should support GEV, U3V or both device types on this system
echo ""
echo "Should mvGenTL_Acquire support GEV devices, such as mvBlueCOUGAR (default is 'yes')?"
echo "Hit 'n' + <Enter> for 'no', or just <Enter> for 'yes'."
read YES_NO
if [ "$YES_NO" == "n" ] || [ "$YES_NO" == "N" ]; then
    GEV_SUPPORT=FALSE
else
    GEV_SUPPORT=TRUE
fi
echo ""
echo "Should mvGenTL_Acquire support U3V devices, such as mvBlueFOX3 (default is 'yes')?"
echo "Hit 'n' + <Enter> for 'no', or just <Enter> for 'yes'."
read YES_NO
if [ "$YES_NO" == "n" ] || [ "$YES_NO" == "N" ]; then
    U3V_SUPPORT=FALSE
else
    U3V_SUPPORT=TRUE
fi
 
# Create the *.conf files if the system is supporting ld.so.conf.d
if grep -q '/etc/ld.so.conf.d/' /etc/ld.so.conf; then
  sudo rm -f $GENICAM_LDSOCONF_FILE; sudo touch $GENICAM_LDSOCONF_FILE
  sudo rm -f $ACQUIRE_LDSOCONF_FILE; sudo touch $ACQUIRE_LDSOCONF_FILE
fi

# Create the export files if the system is supporting profile.d
if grep -q '/etc/profile.d/' /etc/profile; then
  sudo rm -f $GENICAM_EXPORT_FILE; sudo touch $GENICAM_EXPORT_FILE
  sudo rm -f $ACQUIRE_EXPORT_FILE; sudo touch $ACQUIRE_EXPORT_FILE
fi

# Check if the destination directory exist, else create it
if ! [ -d $DEF_DIRECTORY ]; then
  # the destination directory does not yet exist
  # first try to create it as a normal user
  mkdir -p $DEF_DIRECTORY >/dev/null 2>&1
  if ! [ -d $DEF_DIRECTORY ]; then
    # that didn't work
    # now try it as superuser
    sudo mkdir -p $DEF_DIRECTORY
  fi
  if ! [ -d $DEF_DIRECTORY  ]; then
    echo 'ERROR: Could not create target directory' $DEF_DIRECTORY '.'
    echo 'Problem:'$?
    echo 'Maybe you specified a partition that was mounted read only?'
    echo
    exit
  fi
else
  echo 'Installation directory already exists.'
fi

# in case the directory already existed BUT it belongs to other user
sudo chown -R $USER:$USER $DEF_DIRECTORY

# Check the actual tarfile
if ! [ -r $TARFILE ]; then
  echo 'ERROR: could not read' $TARFILE.
  echo
  exit
fi

# needed at compile time (used during development, but not shipped with the final program)
#ACT=$API-$VERSION.tar

# needed at run time
BC=mvGenTL_Acquire_runtime
BCT=$BC-$VERSION.tar

# Now unpack the tar-file into the target directory
cd /tmp
rm -rf mvIMPACT_Acquire-ARM*
tar xfz "$SCRIPTSOURCEDIR/$TARFILE"
cd $ACT
cp -R -d * $DEF_DIRECTORY

if ! [ -r $GENICAM_EXPORT_FILE ]; then
   echo 'Error : cannot write to' $GENICAM_EXPORT_FILE.
   echo 'After the next boot, the required environment variables will not be set.'
   echo
else
   # tests below do not yet check for *commented out* export lines
   if grep -q 'GENICAM_ROOT=' $GENICAM_EXPORT_FILE; then
      echo 'GENICAM_ROOT already defined in' $GENICAM_EXPORT_FILE.
   else
      sudo sh -c "echo 'export GENICAM_ROOT=$DEF_DIRECTORY/runtime' >> $GENICAM_EXPORT_FILE"
   fi
   if grep -q 'GENICAM_ROOT_V$GENICAM_VERSION=' $GENICAM_EXPORT_FILE; then
      echo 'GENICAM_ROOT_V'$GENICAM_VERSION' already defined in' $GENICAM_EXPORT_FILE.
   else
      sudo sh -c "echo 'export GENICAM_ROOT_V$GENICAM_VERSION=$DEF_DIRECTORY/runtime' >> $GENICAM_EXPORT_FILE"
   fi

   if grep -q 'GENICAM_GENTL32_PATH=' $GENICAM_EXPORT_FILE; then
      echo 'GENICAM_GENTL32_PATH already defined in' $GENICAM_EXPORT_FILE.
   else
      sudo sh -c "echo 'if [ x\$GENICAM_GENTL32_PATH == x ]; then
   export GENICAM_GENTL32_PATH=$DEF_DIRECTORY/lib/$TARGET_UNCAPITALIZED
elif [ x\$GENICAM_GENTL32_PATH != x$DEF_DIRECTORY/lib/$TARGET_UNCAPITALIZED ]; then
   if ! \$(echo \$GENICAM_GENTL32_PATH | grep -q \":$DEF_DIRECTORY/lib/$TARGET_UNCAPITALIZED\"); then
      export GENICAM_GENTL32_PATH=\$GENICAM_GENTL32_PATH:$DEF_DIRECTORY/lib/$TARGET_UNCAPITALIZED
   fi
fi' >> $GENICAM_EXPORT_FILE"
   fi

# Since mvImpact Acquire version 2.7.0, version 2.4 of the GenICam cache should be able to coexist with
# version 2.3, however they must point to different folders!
# Since mvIMPACT Acquire version 2.14.0, version 3.0 of the GenICam cache should be able to coexist with
# version 2.3 and 2.4, however they must point to different folders!
   if grep -q 'GENICAM_CACHE_V2_3=' $GENICAM_EXPORT_FILE; then
      echo 'GENICAM_CACHE_V2_3 already defined in' $GENICAM_EXPORT_FILE.
   fi
   if grep -q 'GENICAM_CACHE_V2_4=' $GENICAM_EXPORT_FILE; then
      echo 'GENICAM_CACHE_V2_4 already defined in' $GENICAM_EXPORT_FILE.
   fi
   if grep -q 'GENICAM_CACHE_V3_0=' $GENICAM_EXPORT_FILE; then
      echo 'GENICAM_CACHE_V3_0 already defined in' $GENICAM_EXPORT_FILE.
   else
      sudo mkdir -p $DEF_DIRECTORY/runtime/cache/v3_0
      sudo sh -c "echo 'export GENICAM_CACHE_V3_0='$DEF_DIRECTORY'/runtime/cache/v3_0' >> $GENICAM_EXPORT_FILE"
   fi
   if grep -q 'GENICAM_LOG_CONFIG_V'$GENICAM_VERSION'=' $GENICAM_EXPORT_FILE; then
      echo 'GENICAM_LOG_CONFIG_V'$GENICAM_VERSION' already defined in' $GENICAM_EXPORT_FILE.
   else
      sudo sh -c "echo 'export GENICAM_LOG_CONFIG_V'$GENICAM_VERSION'=$DEF_DIRECTORY/runtime/log/config-unix/DefaultLogging.properties' >> $GENICAM_EXPORT_FILE"
   fi
fi

#Set the necessary exports and library paths
if grep -q 'MVIMPACT_ACQUIRE_DIR=' $ACQUIRE_EXPORT_FILE; then
   echo 'MVIMPACT_ACQUIRE_DIR already defined in' $ACQUIRE_EXPORT_FILE.
else
   sudo sh -c "echo 'export MVIMPACT_ACQUIRE_DIR=$DEF_DIRECTORY' >> $ACQUIRE_EXPORT_FILE"
fi

if grep -q "$DEF_DIRECTORY/lib/$TARGET_UNCAPITALIZED" $ACQUIRE_LDSOCONF_FILE; then
   echo "$DEF_DIRECTORY/lib/$TARGET_UNCAPITALIZED already defined in" $ACQUIRE_LDSOCONF_FILE.
else
   sudo sh -c "echo '$DEF_DIRECTORY/lib/$TARGET_UNCAPITALIZED' >> $ACQUIRE_LDSOCONF_FILE"
fi

if grep -q "$DEF_DIRECTORY/Toolkits/expat/bin/$TARGET_UNCAPITALIZED/lib" $ACQUIRE_LDSOCONF_FILE; then
   echo "$DEF_DIRECTORY/Toolkits/expat/bin/$TARGET_UNCAPITALIZED/lib already defined in" $ACQUIRE_LDSOCONF_FILE.
else
   sudo sh -c "echo '$DEF_DIRECTORY/Toolkits/expat/bin/$TARGET_UNCAPITALIZED/lib' >> $ACQUIRE_LDSOCONF_FILE"
fi
if [ "$U3V_SUPPORT" == "TRUE" ]; then
   if grep -q "$DEF_DIRECTORY/Toolkits/libudev/bin/$TARGET_UNCAPITALIZED/lib" $ACQUIRE_LDSOCONF_FILE; then
      echo "$DEF_DIRECTORY/Toolkits/libudev/bin/$TARGET_UNCAPITALIZED/lib already defined in" $ACQUIRE_LDSOCONF_FILE.
   else
      sudo sh -c "echo '$DEF_DIRECTORY/Toolkits/libudev/bin/$TARGET_UNCAPITALIZED/lib' >> $ACQUIRE_LDSOCONF_FILE"
   fi
fi

# now do the shared linker setup
if ! [ -r $GENICAM_LDSOCONF_FILE ]; then
   echo 'Error : cannot write to' $GENICAM_LDSOCONF_FILE.
   echo 'Execution will fail, as at run time, the shared objects will not be found.'
   echo
else
   if [ "$TARGET" = "ARMsf" ]; then
      GENILIBPATH=Linux32_ARM
   else
      GENILIBPATH=Linux32_ARMhf
   fi
   # tests below do not check for *commented out* link lines
   # must later add sub-string check

   # acquire libs
   if grep -q "$DEF_DIRECTORY/lib/$TARGET_UNCAPITALIZED" $GENICAM_LDSOCONF_FILE; then
      echo "$DEF_DIRECTORY/lib/$TARGET_UNCAPITALIZED already defined in" $GENICAM_LDSOCONF_FILE.
   else
      sudo sh -c "echo '$DEF_DIRECTORY/lib/$TARGET_UNCAPITALIZED' >> $GENICAM_LDSOCONF_FILE"
   fi

   # GenICam libs
   if grep -q "$DEF_DIRECTORY/runtime/bin/$GENILIBPATH" $GENICAM_LDSOCONF_FILE; then
      echo "$DEF_DIRECTORY/runtime/bin/$GENILIBPATH already defined in" $GENICAM_LDSOCONF_FILE.
   else
      sudo sh -c "echo '$DEF_DIRECTORY/runtime/bin/$GENILIBPATH' >> $GENICAM_LDSOCONF_FILE"
   fi
   if grep -q "$DEF_DIRECTORY/runtime/bin/$GENILIBPATH/GenApi/Generic" $GENICAM_LDSOCONF_FILE; then
      echo "$DEF_DIRECTORY/runtime/bin/$GENILIBPATH/GenApi/Generic already defined in" $GENICAM_LDSOCONF_FILE.
   else
      sudo sh -c "echo '$DEF_DIRECTORY/runtime/bin/$GENILIBPATH/GenApi/Generic' >> $GENICAM_LDSOCONF_FILE"
   fi
fi

# This variable must be exported, or else wxPropView-related make problems can arise
export MVIMPACT_ACQUIRE_DIR=$DEF_DIRECTORY

#create softlinks for the Toolkits libraries
createSoftlink $DEF_DIRECTORY/Toolkits/expat/bin/$TARGET_UNCAPITALIZED/lib libexpat.so.1.5.2 libexpat.so.0
createSoftlink $DEF_DIRECTORY/Toolkits/expat/bin/$TARGET_UNCAPITALIZED/lib libexpat.so.0 libexpat.so
createSoftlink $DEF_DIRECTORY/Toolkits/libusb-1.0.20/bin/$TARGET_UNCAPITALIZED/lib libusb-1.0.so.0.1.0  libusb-1.0.so.0
createSoftlink $DEF_DIRECTORY/Toolkits/libusb-1.0.20/bin/$TARGET_UNCAPITALIZED/lib libusb-1.0.so.0  libusb-1.0.so
createSoftlink $DEF_DIRECTORY/Toolkits/libudev/bin/$TARGET_UNCAPITALIZED/lib libudev.so.0.13.0 libudev.so.0
createSoftlink $DEF_DIRECTORY/Toolkits/libudev/bin/$TARGET_UNCAPITALIZED/lib libudev.so.0 libudev.so

# Since the native make target will not be 'armhf' or 'armsf', a softlink has to be created,
# or else the mv apps and tools will not be able to be linked with the mv libraries.
createSoftlink $DEF_DIRECTORY/lib $TARGET_UNCAPITALIZED $ARM_ARCHITECTURE
createSoftlink $DEF_DIRECTORY/Toolkits/expat/bin $TARGET_UNCAPITALIZED $ARM_ARCHITECTURE
createSoftlink $DEF_DIRECTORY/Toolkits/libudev/bin $TARGET_UNCAPITALIZED $ARM_ARCHITECTURE
createSoftlink $DEF_DIRECTORY/Toolkits/libusb-1.0.20/bin $TARGET_UNCAPITALIZED $ARM_ARCHITECTURE
createSoftlink $DEF_DIRECTORY/Toolkits/FreeImage3160/bin/Release/FreeImage $TARGET_UNCAPITALIZED $ARM_ARCHITECTURE

# Set up the fresh libs with ldconfig
sudo /sbin/ldconfig

#Set the necessary cti softlink
createSoftlink $DEF_DIRECTORY/lib/$TARGET_UNCAPITALIZED libmvGenTLProducer.so mvGenTLProducer.cti

# In case GEV devices should not be supported remove mvIPConfigure 
if [ "$GEV_SUPPORT" == "FALSE" ]; then
    if [ -d $DEF_DIRECTORY/apps/mvIPConfigure ] && [ -r $DEF_DIRECTORY/apps/mvIPConfigure/Makefile ]; then
        sudo rm -rf $DEF_DIRECTORY/apps/mvIPConfigure
    fi
fi

# Ask whether the samples should be built natively
echo
echo "Do you want the sample applications to be built (default is 'yes')?"
echo "A native g++ compiler has to be present on the system!"
echo "Hit 'n' + <Enter> for 'no', or just <Enter> for 'yes'."
read YES_NO
if [ "$YES_NO" == "n" ] || [ "$YES_NO" == "N" ]; then
  echo 'The tools and samples were not built.'
  echo 'To build them yourself, type:'
  echo '  cd '$DEF_DIRECTORY
  echo '  make native'
  echo '  sudo /sbin/ldconfig'
else
  if [ "$(which g++)" != "" ]; then
    echo
    echo "Do you want the GUI tools to be built (default is 'yes')?"
    echo "This requires wxWidgets libraries to be present on your system."
    echo "If they are missing, an attempt will be made to download them."
    echo "Hit 'n' + <Enter> for 'no', or just <Enter> for 'yes'."
    read YES_NO
    if [ "$YES_NO" == "n" ] || [ "$YES_NO" == "N" ]; then
      # remove GUI apps sources since they are not needed
      rm -rf $DEF_DIRECTORY/apps/mv*
      rm -rf $DEF_DIRECTORY/apps/Common/FirmwareUpdate_mvHYPERION
    else
      # check if wxWidgets are present else download them
      if [ "$(wx-config --release 2>&1 | grep -c "^3.")" != "1" ]; then
        if [ "x$(which apt-get)" != "x" ]; then
          echo
          echo "Updating file lists from repositories..."
          echo
          sudo apt-get update
          echo
          echo "Downloading and installing wxWidgets via apt-get..."
          echo
          sudo apt-get -q -y install libwxgtk3.0-dev libwxbase3.0-0 libwxbase3.0-dev libwxgtk3.0-0 wx3.0-headers build-essential libgtk2.0-dev
          echo
          if [ $? == 0 ] && [ "x$(which wx-config)" != "x" ]; then
            echo "Necessary wxWidgets libraries installed successfully!"
            echo
          else
            echo "wxWidgets libraries could not automatically download and install on this system!"
            echo "Please either install wxWidgets libraries manually and re-run this installer script,"
            echo "or re-run this script and choose not to build the wxWidgets GUI Tools altogether!"
            echo
            exit 1
          fi
        else
          echo
          echo "Could not download wxWidgets, apt-get is missing!"
          echo
          echo "wxWidgets libraries could not automatically download and install on this system!"
          echo "Please either install wxWidgets libraries manually and re-run this installer script,"
          echo "or re-run this script and choose not to build the wxWidgets GUI Tools altogether!"
          echo
          exit 1
        fi
      fi
      cd $DEF_DIRECTORY
      sudo /sbin/ldconfig
    fi
    # build all apps and samples.
    echo "Building samples and/or tools..."
    make native
  else
    echo "Sample applications and/or GUI tools cannot be built, as the system is missing a g++ compiler!"
  fi

# Shall the MV Tools be linked in /usr/bin?
   if [ "$GEV_SUPPORT" == "TRUE" ]; then
       echo "Do you want to set a link to /usr/bin for wxPropView, mvIPConfigure and mvDeviceConfigure (default is 'yes')?"
   else
       echo "Do you want to set a link to /usr/bin for wxPropView and mvDeviceConfigure (default is 'yes')?"
   fi
   echo "Hit 'n' + <Enter> for 'no', or just <Enter> for 'yes'."
   read YES_NO
   if [ "$YES_NO" == "n" ] || [ "$YES_NO" == "N" ]; then
      echo "Will not set any new link to /usr/bin."
   else
      if [ -r /usr/bin ]; then
         # Set wxPropView
         if [ -r $DEF_DIRECTORY/apps/mvPropView/$ARM_ARCHITECTURE/wxPropView ]; then
            sudo rm -f /usr/bin/wxPropView
            sudo ln -s $DEF_DIRECTORY/apps/mvPropView/$ARM_ARCHITECTURE/wxPropView /usr/bin/wxPropView
         fi
         # Set mvIPConfigure
         if [ "$GEV_SUPPORT" == "TRUE" ]; then
             if [ -r $DEF_DIRECTORY/apps/mvIPConfigure/$ARM_ARCHITECTURE/mvIPConfigure ]; then
                sudo rm -f /usr/bin/mvIPConfigure
                sudo ln -s $DEF_DIRECTORY/apps/mvIPConfigure/$ARM_ARCHITECTURE/mvIPConfigure /usr/bin/mvIPConfigure
             fi
         fi
         # Set mvDeviceConfigure
         if [ -r $DEF_DIRECTORY/apps/mvDeviceConfigure/$ARM_ARCHITECTURE/mvDeviceConfigure ]; then
            sudo rm -f /usr/bin/mvDeviceConfigure
            sudo ln -s $DEF_DIRECTORY/apps/mvDeviceConfigure/$ARM_ARCHITECTURE/mvDeviceConfigure /usr/bin/mvDeviceConfigure
         fi
      fi
   fi
fi

# copy the mvBF3 boot-device and an universal udev rules file for U3V cameras to the system 
if [ "$U3V_SUPPORT" == "TRUE" ]; then
    echo
    echo "Do you want to copy the necessary files to /etc/udev/rules.d for non-root user support (default is 'yes')?"
    echo "Hit 'n' + <Enter> for 'no', or just <Enter> for 'yes'."
    read YES_NO
    if [ "$YES_NO" == "n" ] || [ "$YES_NO" == "N" ]; then
       echo
       echo 'To grant non-root user support,'
       echo 'copy 52-U3V.rules and 52-mvbf3.rules the file to /etc/udev/rules.d'
       echo
    else
       sudo cp -f $DEF_DIRECTORY/Scripts/52-U3V.rules /etc/udev/rules.d
       sudo cp -f $DEF_DIRECTORY/Scripts/52-mvbf3.rules /etc/udev/rules.d
    fi
fi


# check if plugdev group exists and the user is member of it
if [ "$U3V_SUPPORT" == "TRUE" ]; then
    echo ""
    if ! grep -q plugdev /etc/group; then
       echo "Group 'plugdev' don't exists, this is necessary to run as non-root user, do you want to create it"
       echo "and add users to 'plugdev' (default is 'yes')?"
       echo "Hit 'n' + <Enter> for 'no', or just <Enter> for 'yes'."
       read YES_NO
       if [ "$YES_NO" == "n" ] || [ "$YES_NO" == "N" ]; then
          echo
          echo "'plugdev' will be not created and you can't run the device as non-root user!"
          echo "If you want non-root users support, you will need to create 'plugdev'"
          echo "and add the users to this group."
       else
          sudo /usr/sbin/groupadd -g 46 plugdev
          sudo /usr/sbin/usermod $USER -A plugdev
          echo "Group 'plugdev' created and user '"$USER"' added to it."
       fi
    fi
fi

# create the cache for genicam xml files.
if ! [ -d /etc/matrix-vision/mvimpact-acquire/genicam ]; then
  mkdir -p /etc/matrix-vision/mvimpact-acquire/genicam >/dev/null 2>&1
  if ! [ -d /etc/matrix-vision/mvimpact-acquire/genicam ]; then
      # that didn't work, now try it as superuser
      sudo mkdir -p /etc/matrix-vision/mvimpact-acquire/genicam >/dev/null 2>&1
  fi
  if ! [ -d /etc/matrix-vision/mvimpact-acquire/genicam  ]; then
    echo 'ERROR: Could not create "/etc/matrix-vision/mvimpact-acquire/genicam" directory.'
    echo 'Problem:'$?
    echo 'Maybe you specified a partition that was mounted read only?'
    echo
    exit
  fi
fi

# make sure the complete mvIA-tree and the genicam folder in /etc belongs to the user
sudo chown -R $USER:$USER $DEF_DIRECTORY
sudo chown -R $USER:$USER /etc/matrix-vision

#Check whether the network buffers are configured
if [ "$GEV_SUPPORT" == "TRUE" ]; then
ERROR=0
echo "------------------------------------GEV Check--------------------------------------"
    if [ "$(which sysctl)" == "" ]; then
       echo "Warning: 'sysctl' not present on the system, network parameters cannot be checked!"
       ERROR=1
    else
       RMEM=$(sysctl -n net.core.rmem_max)
       WMEM=$(sysctl -n net.core.wmem_max) 
       BKLG=$(sysctl -n net.core.netdev_max_backlog) 
       if [ $RMEM -lt 12582912 ]; then
           echo "Warning: 'net.core.rmem_max' Receive buffer settings are low($RMEM bytes)!"
           ERROR=1
       fi
       if [ $WMEM -lt 12582912 ]; then
           echo "Warning: 'net.core.wmem_max' Send buffer settings are low($WMEM bytes)!"
           ERROR=1
       fi
       if [ $BKLG -lt 5000 ]; then
           echo "Warning: 'net.core.netdev_max_backlog' input queue settings are low($BKLG elements)!"
           ERROR=1
       fi
       if [ $ERROR == 1 ]; then
           echo "Incomplete frames may occur during image acquisition!"
       fi
   fi
   if [ $ERROR == 1 ]; then
       echo
       echo "Please refer to 'Quickstart/Optimizing the network configuration' section of the "
       echo "User Manual for more information on how to adjust the network buffers"
       echo "http://www.matrix-vision.com/manuals/mvBlueCOUGAR-X/mvBC_page_quickstart.html#mvBC_subsubsection_quickstart_network_configuration_controller"
       echo "-----------------------------------------------------------------------------------"
   else
      echo "                                       OK!                                         "
      echo "-----------------------------------------------------------------------------------"
   fi
fi

#Check whether the USBFS Memory is configured
if [ "$U3V_SUPPORT" == "TRUE" ]; then
ERROR=0
echo "------------------------------------U3V Check--------------------------------------"
    if [ ! -r /sys/module/usbcore/parameters/usbfs_memory_mb ]; then
       echo "Warning: 'usbfs_memory_mb' parameter does not exist or cannot be read!"
       ERROR=1
    else
       USBMEM=$(cat /sys/module/usbcore/parameters/usbfs_memory_mb)
       if [ $USBMEM -lt 64 ]; then
           echo "Warning: 'usbfs_memory_mb' Kernel USB file system buffer settings are low($USBMEM MB)!"
           echo "Incomplete frames may occur during image acquisition!"
           ERROR=1
        fi
   fi
   if [ $ERROR == 1 ]; then
       echo
       echo "Please refer to 'Quickstart/Linux/Optimizing USB performance' section of the "
       echo "User Manual for more information on how to adjust the kernel USB buffers"
       echo "http://www.matrix-vision.com/manuals/mvBlueFOX3/mvBC_page_quickstart.html#mvBC_subsubsection_quickstart_linux_requirements_optimising_usb"
       echo "-----------------------------------------------------------------------------------"
   else
      echo "                                       OK!                                         "
      echo "-----------------------------------------------------------------------------------"
   fi
fi

rm -f $SCRIPTSOURCEDIR/0
source $GENICAM_EXPORT_FILE
echo
echo
echo "-----------------------------------------------------------------------------------"
echo "                            Installation successful!                               "
echo "-----------------------------------------------------------------------------------"
echo

echo 'If the device is not initilising execute the following commands:'
echo '  ' export GENICAM_ROOT=$DEF_DIRECTORY/runtime
echo '  ' export "GENICAM_ROOT_V$GENICAM_VERSION="$DEF_DIRECTORY/runtime
echo '  ' export GENICAM_GENTL32_PATH=$DEF_DIRECTORY/runtime/lib
echo '  ' export "GENICAM_CACHE_V$GENICAM_VERSION="$DEF_DIRECTORY/runtime/cache/v3_0
echo '  ' export "GENICAM_LOG_CONFIG_V$GENICAM_VERSION="$DEF_DIRECTORY/runtime/log/config-unix/DefaultLogging.properties
echo 'or source' $GENICAM_EXPORT_FILE:
echo '  ' . $GENICAM_EXPORT_FILE
echo 'or restart X11 (Ctrl-Alt-Backspace (twice on OpenSuSE)).'
echo '(The exports above have been added to' $GENICAM_EXPORT_FILE.
echo ' This file is read when you log in, which is normally done when starting X11.)'
echo
